@@ -28,6 +28,7 @@
 #endif
 
 #include <libdivecomputer/hw_ostc.h>
+#include <libdivecomputer/hw_ostc3.h>
 #include "libdivecomputer/units.h"
 
 #include "context-private.h"
@@ -67,6 +68,9 @@
 
 #define OSTC3_ZHL16    0
 #define OSTC3_ZHL16_GF 1
+#define OSTC4_VPM      2
+
+#define OSTC4      0x3B
 
 #define UNSUPPORTED 0xFFFFFFFF
 
@@ -87,7 +91,7 @@ typedef struct hw_ostc_layout_t {
 	unsigned int temperature;
 	unsigned int battery;
 	unsigned int desat;
-	unsigned int fw_version;
+	unsigned int firmware;
 	unsigned int deco_info1;
 	unsigned int deco_info2;
 	unsigned int decomode;
@@ -100,7 +104,8 @@ typedef struct hw_ostc_gasmix_t {
 
 typedef struct hw_ostc_parser_t {
 	dc_parser_t base;
-	unsigned int frog;
+	unsigned int hwos;
+	unsigned int model;
 	unsigned int serial;
 	// Cached fields.
 	unsigned int cached;
@@ -139,7 +144,7 @@ static const hw_ostc_layout_t hw_ostc_layout_ostc = {
 	13, /* temperature */
 	34, /* battery volt after dive */
 	17, /* desat */
-	32, /* fw_version */
+	32, /* firmware */
 	49, /* deco_info1 */
 	50, /* deco_info1 */
 	51, /* decomode */
@@ -156,7 +161,7 @@ static const hw_ostc_layout_t hw_ostc_layout_frog = {
 	19, /* temperature */
 	34, /* battery volt after dive */
 	23, /* desat */
-	32, /* fw_version */
+	32, /* firmware */
 	49, /* deco_info1 */
 	50, /* deco_info2 */
 	51, /* decomode */
@@ -173,7 +178,7 @@ static const hw_ostc_layout_t hw_ostc_layout_ostc3 = {
 	22, /* temperature */
 	50, /* battery volt after dive */
 	26, /* desat */
-	48, /* fw_version */
+	48, /* firmware */
 	77, /* deco_info1 */
 	78, /* deco_info2 */
 	79, /* decomode */
@@ -217,7 +222,7 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 	}
 
 	// Check the profile version
-	unsigned int version = data[parser->frog ? 8 : 2];
+	unsigned int version = data[parser->hwos ? 8 : 2];
 	const hw_ostc_layout_t *layout = NULL;
 	unsigned int header = 0;
 	switch (version) {
@@ -303,7 +308,7 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 }
 
 dc_status_t
-hw_ostc_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int serial, unsigned int frog)
+hw_ostc_parser_create_internal (dc_parser_t **out, dc_context_t *context, unsigned int serial, unsigned int hwos, unsigned int model)
 {
 	hw_ostc_parser_t *parser = NULL;
 
@@ -318,7 +323,8 @@ hw_ostc_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int se
 	}
 
 	// Set the default values.
-	parser->frog = frog;
+	parser->hwos = hwos;
+	parser->model = model;
 	parser->cached = 0;
 	parser->version = 0;
 	parser->header = 0;
@@ -338,6 +344,18 @@ hw_ostc_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int se
 }
 
 
+dc_status_t
+hw_ostc_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int serial, unsigned int hwos)
+{
+	return hw_ostc_parser_create_internal (out, context, serial, hwos, 0);
+}
+
+dc_status_t
+hw_ostc3_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int serial, unsigned int model)
+{
+	return hw_ostc_parser_create_internal (out, context, serial, 1, model);
+}
+
 static dc_status_t
 hw_ostc_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
@@ -551,9 +569,21 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 				snprintf(buf, BUFLEN, "%0u:%02u", array_uint16_le (data + layout->desat) / 60,
 						 array_uint16_le (data + layout->desat) % 60);
 				break;
-			case 2: /* fw_version */
+			case 2: /* firmware */
 				string->desc = "FW Version";
-				snprintf(buf, BUFLEN, "%0u.%02u", data[layout->fw_version], data[layout->fw_version + 1]);
+				/* OSTC4 stores firmware as XXXX XYYY YYZZ ZZZB, -> X.Y.Z beta? */
+				if (parser->model == OSTC4) {
+					int firmwareOnDevice = array_uint16_le (data + layout->firmware);
+					unsigned char X = 0, Y = 0, Z = 0, beta = 0;
+					X = (firmwareOnDevice & 0xF800) >> 11;
+					Y = (firmwareOnDevice & 0x07C0) >> 6;
+					Z = (firmwareOnDevice & 0x003E) >> 1;
+					beta = firmwareOnDevice & 0x0001;
+
+					snprintf(buf, BUFLEN, "%u.%u.%u%s\n", X, Y, Z, beta? "beta": "");
+				} else {
+					snprintf(buf, BUFLEN, "%0u.%02u", data[layout->firmware], data[layout->firmware + 1]);
+				}
 				break;
 			case 3: /* serial */
 				string->desc = "Serial";
@@ -561,24 +591,26 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 				break;
 			case 4: /* Deco model */
 				string->desc = "Deco model";
-				if ((version == 0x23 && data[layout->decomode] == OSTC3_ZHL16) ||
+				if (((version == 0x23 || version == 0x24) && data[layout->decomode] == OSTC3_ZHL16) ||
 						(version == 0x22 && data[layout->decomode] == FROG_ZHL16) ||
 						(version == 0x21 && (data[layout->decomode] == OSTC_ZHL16_OC || data[layout->decomode] == OSTC_ZHL16_CC)))
 					strncpy(buf, "ZH-L16", BUFLEN);
-				if ((version == 0x23 && data[layout->decomode] == OSTC3_ZHL16_GF) ||
+				else if (((version == 0x23 || version == 0x24) && data[layout->decomode] == OSTC3_ZHL16_GF) ||
 						(version == 0x22 && data[layout->decomode] == FROG_ZHL16_GF) ||
 						(version == 0x21 && (data[layout->decomode] == OSTC_ZHL16_OC_GF || data[layout->decomode] == OSTC_ZHL16_CC_GF)))
 					strncpy(buf, "ZH-L16-GF", BUFLEN);
+				else if (((version == 0x24) && data[layout->decomode] == OSTC4_VPM))
+					strncpy(buf, "VPM", BUFLEN);
 				else
 					return DC_STATUS_DATAFORMAT;
 				break;
 			case 5: /* Deco model info */
 				string->desc = "Deco model info";
-				if ((version == 0x23 && data[layout->decomode] == OSTC3_ZHL16) ||
+				if (((version == 0x23 || version == 0x24) && data[layout->decomode] == OSTC3_ZHL16) ||
 						(version == 0x22 && data[layout->decomode] == FROG_ZHL16) ||
 						(version == 0x21 && (data[layout->decomode] == OSTC_ZHL16_OC || data[layout->decomode] == OSTC_ZHL16_CC)))
 					snprintf(buf, BUFLEN, "Saturation %u, Desaturation %u", layout->deco_info1, layout->deco_info2);
-				if ((version == 0x23 && data[layout->decomode] == OSTC3_ZHL16_GF) ||
+				else if (((version == 0x23 || version == 0x24) && data[layout->decomode] == OSTC3_ZHL16_GF) ||
 						(version == 0x22 && data[layout->decomode] == FROG_ZHL16_GF) ||
 						(version == 0x21 && (data[layout->decomode] == OSTC_ZHL16_OC_GF || data[layout->decomode] == OSTC_ZHL16_CC_GF)))
 					snprintf(buf, BUFLEN, "GF %u/%u", data[layout->deco_info1], data[layout->deco_info2]);
@@ -681,6 +713,14 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 		}
 	}
 
+	// Get the firmware version.
+	unsigned int firmware = 0;
+	if (parser->model == OSTC4) {
+		firmware = array_uint16_le (data + layout->firmware);
+	} else {
+		firmware = array_uint16_be (data + layout->firmware);
+	}
+
 	unsigned int time = 0;
 	unsigned int nsamples = 0;
 
@@ -901,6 +941,10 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 					if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 					break;
 				case 1: // Deco / NDL
+					// Due to a firmware bug, the deco/ndl info is incorrect for
+					// all OSTC4 dives with a firmware older than version 1.0.8.
+					if (parser->model == OSTC4 && firmware < 0x0810)
+						break;
 					if (data[offset]) {
 						sample.deco.type = DC_DECO_DECOSTOP;
 						sample.deco.depth = data[offset];
