@@ -39,18 +39,31 @@
 #endif
 
 #include "irda.h"
+#include "common-private.h"
 #include "context-private.h"
 #include "array.h"
 
 #ifdef _WIN32
+typedef int s_ssize_t;
+typedef DWORD s_errcode_t;
 #define S_ERRNO WSAGetLastError ()
 #define S_EAGAIN WSAEWOULDBLOCK
+#define S_ENOMEM WSA_NOT_ENOUGH_MEMORY
+#define S_EINVAL WSAEINVAL
+#define S_EACCES WSAEACCES
+#define S_EAFNOSUPPORT WSAEAFNOSUPPORT
 #define S_INVALID INVALID_SOCKET
 #define S_IOCTL ioctlsocket
 #define S_CLOSE closesocket
 #else
+typedef ssize_t s_ssize_t;
+typedef int s_errcode_t;
 #define S_ERRNO errno
 #define S_EAGAIN EAGAIN
+#define S_ENOMEM ENOMEM
+#define S_EINVAL EINVAL
+#define S_EACCES EACCES
+#define S_EAFNOSUPPORT EAFNOSUPPORT
 #define S_INVALID -1
 #define S_IOCTL ioctl
 #define S_CLOSE close
@@ -60,32 +73,47 @@
 #define snprintf _snprintf
 #endif
 
-struct irda_t {
+struct dc_irda_t {
 	dc_context_t *context;
 #ifdef _WIN32
 	SOCKET fd;
 #else
 	int fd;
 #endif
-	long timeout;
+	int timeout;
 };
 
-
-int
-irda_socket_open (irda_t **out, dc_context_t *context)
+static dc_status_t
+syserror(s_errcode_t errcode)
 {
+	switch (errcode) {
+	case S_EINVAL:
+		return DC_STATUS_INVALIDARGS;
+	case S_ENOMEM:
+		return DC_STATUS_NOMEMORY;
+	case S_EACCES:
+		return DC_STATUS_NOACCESS;
+	case S_EAFNOSUPPORT:
+		return DC_STATUS_UNSUPPORTED;
+	default:
+		return DC_STATUS_IO;
+	}
+}
+
+dc_status_t
+dc_irda_open (dc_irda_t **out, dc_context_t *context)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_irda_t *device = NULL;
+
 	if (out == NULL)
-		return -1; // EINVAL (Invalid argument)
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	irda_t *device = (irda_t *) malloc (sizeof (irda_t));
+	device = (dc_irda_t *) malloc (sizeof (dc_irda_t));
 	if (device == NULL) {
-#ifdef _WIN32
-		SYSERROR (context, ERROR_OUTOFMEMORY);
-#else
-		SYSERROR (context, ENOMEM);
-#endif
-		return -1; // ENOMEM (Not enough space)
+		SYSERROR (context, S_ENOMEM);
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Library context.
@@ -101,6 +129,7 @@ irda_socket_open (irda_t **out, dc_context_t *context)
 	int rc = WSAStartup (wVersionRequested, &wsaData);
 	if (rc != 0) {
 		SYSERROR (context, rc);
+		status = DC_STATUS_UNSUPPORTED;
 		goto error_free;
 	}
 
@@ -110,6 +139,7 @@ irda_socket_open (irda_t **out, dc_context_t *context)
 	if (LOBYTE (wsaData.wVersion) != 2 ||
 		HIBYTE (wsaData.wVersion) != 2) {
 		ERROR (context, "Incorrect winsock version.");
+		status = DC_STATUS_UNSUPPORTED;
 		goto error_wsacleanup;
 	}
 #endif
@@ -117,13 +147,15 @@ irda_socket_open (irda_t **out, dc_context_t *context)
 	// Open the socket.
 	device->fd = socket (AF_IRDA, SOCK_STREAM, 0);
 	if (device->fd == S_INVALID) {
-		SYSERROR (context, S_ERRNO);
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (context, errcode);
+		status = syserror(errcode);
 		goto error_wsacleanup;
 	}
 
 	*out = device;
 
-    return 0;
+    return DC_STATUS_SUCCESS;
 
 error_wsacleanup:
 #ifdef _WIN32
@@ -131,53 +163,53 @@ error_wsacleanup:
 error_free:
 #endif
 	free (device);
-	return -1;
+	return status;
 }
 
-
-int
-irda_socket_close (irda_t *device)
+dc_status_t
+dc_irda_close (dc_irda_t *device)
 {
-	int errcode = 0;
+	dc_status_t status = DC_STATUS_SUCCESS;
 
 	if (device == NULL)
-		return -1;
+		return DC_STATUS_SUCCESS;
 
 	// Terminate all send and receive operations.
 	shutdown (device->fd, 0);
 
 	// Close the socket.
 	if (S_CLOSE (device->fd) != 0) {
-		SYSERROR (device->context, S_ERRNO);
-		errcode = -1;
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (device->context, errcode);
+		dc_status_set_error(&status, syserror(errcode));
 	}
 
 #ifdef _WIN32
 	// Terminate the winsock dll.
 	if (WSACleanup () != 0) {
-		SYSERROR (device->context, S_ERRNO);
-		errcode = -1;
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (device->context, errcode);
+		dc_status_set_error(&status, syserror(errcode));
 	}
 #endif
 
 	// Free memory.
 	free (device);
 
-	return errcode;
+	return status;
 }
 
-
-int
-irda_socket_set_timeout (irda_t *device, long timeout)
+dc_status_t
+dc_irda_set_timeout (dc_irda_t *device, int timeout)
 {
 	if (device == NULL)
-		return -1; // EINVAL (Invalid argument)
+		return DC_STATUS_INVALIDARGS;
 
-	INFO (device->context, "Timeout: value=%li", timeout);
+	INFO (device->context, "Timeout: value=%i", timeout);
 
 	device->timeout = timeout;
 
-	return 0;
+	return DC_STATUS_SUCCESS;
 }
 
 
@@ -192,11 +224,11 @@ irda_socket_set_timeout (irda_t *device, long timeout)
 				sizeof (struct irda_device_info) * (DISCOVER_MAX_DEVICES - 1)
 #endif
 
-int
-irda_socket_discover (irda_t *device, irda_callback_t callback, void *userdata)
+dc_status_t
+dc_irda_discover (dc_irda_t *device, dc_irda_callback_t callback, void *userdata)
 {
 	if (device == NULL)
-		return -1;
+		return DC_STATUS_INVALIDARGS;
 
 	unsigned char data[DISCOVER_BUFSIZE] = {0};
 #ifdef _WIN32
@@ -221,15 +253,16 @@ irda_socket_discover (irda_t *device, irda_callback_t callback, void *userdata)
 		// discovered, while on Windows it succeeds and sets the number
 		// of devices to zero. Both situations are handled the same here.
 		if (rc != 0) {
-			if (S_ERRNO != S_EAGAIN) {
-				SYSERROR (device->context, S_ERRNO);
-				return -1; // Error during getsockopt call.
+			s_errcode_t errcode = S_ERRNO;
+			if (errcode != S_EAGAIN) {
+				SYSERROR (device->context, errcode);
+				return syserror(errcode);
 			}
 		}
 
 		// Abort if the maximum number of retries is reached.
 		if (nretries++ >= DISCOVER_MAX_RETRIES)
-			return 0;
+			return DC_STATUS_SUCCESS;
 
 		// Restore the size parameter in case it was
 		// modified by the previous getsockopt call.
@@ -266,15 +299,14 @@ irda_socket_discover (irda_t *device, irda_callback_t callback, void *userdata)
 		}
 	}
 
-	return 0;
+	return DC_STATUS_SUCCESS;
 }
 
-
-int
-irda_socket_connect_name (irda_t *device, unsigned int address, const char *name)
+dc_status_t
+dc_irda_connect_name (dc_irda_t *device, unsigned int address, const char *name)
 {
 	if (device == NULL)
-		return -1;
+		return DC_STATUS_INVALIDARGS;
 
 	INFO (device->context, "Connect: address=%08x, name=%s", address, name ? name : "");
 
@@ -300,18 +332,19 @@ irda_socket_connect_name (irda_t *device, unsigned int address, const char *name
 #endif
 
 	if (connect (device->fd, (struct sockaddr *) &peer, sizeof (peer)) != 0) {
-		SYSERROR (device->context, S_ERRNO);
-		return -1;
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (device->context, errcode);
+		return syserror(errcode);
 	}
 
-	return 0;
+	return DC_STATUS_SUCCESS;
 }
 
-int
-irda_socket_connect_lsap (irda_t *device, unsigned int address, unsigned int lsap)
+dc_status_t
+dc_irda_connect_lsap (dc_irda_t *device, unsigned int address, unsigned int lsap)
 {
 	if (device == NULL)
-		return -1;
+		return DC_STATUS_INVALIDARGS;
 
 	INFO (device->context, "Connect: address=%08x, lsap=%u", address, lsap);
 
@@ -332,19 +365,19 @@ irda_socket_connect_lsap (irda_t *device, unsigned int address, unsigned int lsa
 #endif
 
 	if (connect (device->fd, (struct sockaddr *) &peer, sizeof (peer)) != 0) {
-		SYSERROR (device->context, S_ERRNO);
-		return -1;
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (device->context, errcode);
+		return syserror(errcode);
 	}
 
-	return 0;
+	return DC_STATUS_SUCCESS;
 }
 
-
-int
-irda_socket_available (irda_t *device)
+dc_status_t
+dc_irda_get_available (dc_irda_t *device, size_t *value)
 {
 	if (device == NULL)
-		return -1; // EINVAL (Invalid argument)
+		return DC_STATUS_INVALIDARGS;
 
 #ifdef _WIN32
 	unsigned long bytes = 0;
@@ -353,19 +386,27 @@ irda_socket_available (irda_t *device)
 #endif
 
 	if (S_IOCTL (device->fd, FIONREAD, &bytes) != 0) {
-		SYSERROR (device->context, S_ERRNO);
-		return -1;
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (device->context, errcode);
+		return syserror(errcode);
 	}
 
-	return bytes;
+	if (value)
+		*value = bytes;
+
+	return DC_STATUS_SUCCESS;
 }
 
-
-int
-irda_socket_read (irda_t *device, void *data, unsigned int size)
+dc_status_t
+dc_irda_read (dc_irda_t *device, void *data, size_t size, size_t *actual)
 {
-	if (device == NULL)
-		return -1; // EINVAL (Invalid argument)
+	dc_status_t status = DC_STATUS_SUCCESS;
+	size_t nbytes = 0;
+
+	if (device == NULL) {
+		status = DC_STATUS_INVALIDARGS;
+		goto out;
+	}
 
 	struct timeval tv;
 	if (device->timeout >= 0) {
@@ -377,20 +418,23 @@ irda_socket_read (irda_t *device, void *data, unsigned int size)
 	FD_ZERO (&fds);
 	FD_SET (device->fd, &fds);
 
-	unsigned int nbytes = 0;
 	while (nbytes < size) {
 		int rc = select (device->fd + 1, &fds, NULL, NULL, (device->timeout >= 0 ? &tv : NULL));
 		if (rc < 0) {
-			SYSERROR (device->context, S_ERRNO);
-			return -1; // Error during select call.
+			s_errcode_t errcode = S_ERRNO;
+			SYSERROR (device->context, errcode);
+			status = syserror(errcode);
+			goto out;
 		} else if (rc == 0) {
 			break; // Timeout.
 		}
 
-		int n = recv (device->fd, (char*) data + nbytes, size - nbytes, 0);
+		s_ssize_t n = recv (device->fd, (char*) data + nbytes, size - nbytes, 0);
 		if (n < 0) {
-			SYSERROR (device->context, S_ERRNO);
-			return -1; // Error during recv call.
+			s_errcode_t errcode = S_ERRNO;
+			SYSERROR (device->context, errcode);
+			status = syserror(errcode);
+			goto out;
 		} else if (n == 0) {
 			break; // EOF reached.
 		}
@@ -398,30 +442,51 @@ irda_socket_read (irda_t *device, void *data, unsigned int size)
 		nbytes += n;
 	}
 
+	if (nbytes != size) {
+		status = DC_STATUS_TIMEOUT;
+	}
+
+out:
 	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Read", (unsigned char *) data, nbytes);
 
-	return nbytes;
+	if (actual)
+		*actual = nbytes;
+
+	return status;
 }
 
-
-int
-irda_socket_write (irda_t *device, const void *data, unsigned int size)
+dc_status_t
+dc_irda_write (dc_irda_t *device, const void *data, size_t size, size_t *actual)
 {
-	if (device == NULL)
-		return -1; // EINVAL (Invalid argument)
+	dc_status_t status = DC_STATUS_SUCCESS;
+	size_t nbytes = 0;
+
+	if (device == NULL) {
+		status = DC_STATUS_INVALIDARGS;
+		goto out;
+	}
 
-	unsigned int nbytes = 0;
 	while (nbytes < size) {
-		int n = send (device->fd, (char*) data + nbytes, size - nbytes, 0);
+		s_ssize_t n = send (device->fd, (char*) data + nbytes, size - nbytes, 0);
 		if (n < 0) {
-			SYSERROR (device->context, S_ERRNO);
-			return -1; // Error during send call.
+			s_errcode_t errcode = S_ERRNO;
+			SYSERROR (device->context, errcode);
+			status = syserror(errcode);
+			goto out;
 		}
 
 		nbytes += n;
 	}
 
+	if (nbytes != size) {
+		status = DC_STATUS_TIMEOUT;
+	}
+
+out:
 	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Write", (unsigned char *) data, nbytes);
 
-	return nbytes;
+	if (actual)
+		*actual = nbytes;
+
+	return status;
 }
