@@ -69,15 +69,21 @@
 #define WISDOM3     0x4458
 #define A300        0x445A
 #define TX1         0x4542
+#define MUNDIAL2    0x4543
 #define AMPHOS      0x4545
 #define AMPHOSAIR   0x4546
 #define PROPLUS3    0x4548
 #define F11A        0x4549
 #define OCI         0x454B
 #define A300CS      0x454C
+#define MUNDIAL3    0x4550
 #define F11B        0x4554
+#define XPAIR       0x4555
+#define VISION      0x4556
 #define VTX         0x4557
+#define I300        0x4559
 #define I450T       0x4641
+#define I550T       0x4642
 
 #define NORMAL   0
 #define GAUGE    1
@@ -148,7 +154,8 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		model == VEO20 || model == VEO30 ||
 		model == OCS || model == PROPLUS3 ||
 		model == A300 || model == MANTA ||
-		model == INSIGHT2 || model == ZEN) {
+		model == INSIGHT2 || model == ZEN ||
+		model == I300 || model == I550T) {
 		parser->headersize -= PAGESIZE;
 	} else if (model == VT4 || model == VT41) {
 		parser->headersize += PAGESIZE;
@@ -156,7 +163,7 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		parser->headersize += 2 * PAGESIZE;
 	} else if (model == ATOM1) {
 		parser->headersize -= 2 * PAGESIZE;
-	} else if (model == F10) {
+	} else if (model == F10 || model == MUNDIAL2 || model == MUNDIAL3) {
 		parser->headersize = 3 * PAGESIZE;
 		parser->footersize = 0;
 	} else if (model == F11A || model == F11B) {
@@ -214,7 +221,8 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 
 	unsigned int header = 8;
 	if (parser->model == F10 || parser->model == F11A ||
-		parser->model == F11B)
+		parser->model == F11B || parser->model == MUNDIAL2 ||
+		parser->model == MUNDIAL3)
 		header = 32;
 
 	if (abstract->size < header)
@@ -237,6 +245,9 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case ATOM31:
 		case A300AI:
 		case OCI:
+		case I550T:
+		case VISION:
+		case XPAIR:
 			datetime->year   = ((p[5] & 0xE0) >> 5) + ((p[7] & 0xE0) >> 2) + 2000;
 			datetime->month  = (p[3] & 0x0F);
 			datetime->day    = ((p[0] & 0x80) >> 3) + ((p[3] & 0xF0) >> 4);
@@ -253,6 +264,8 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case PROPLUS3:
 		case DATAMASK:
 		case COMPUMASK:
+		case INSIGHT2:
+		case I300:
 			datetime->year   = ((p[3] & 0xE0) >> 1) + (p[4] & 0x0F) + 2000;
 			datetime->month  = (p[4] & 0xF0) >> 4;
 			datetime->day    = p[3] & 0x1F;
@@ -272,6 +285,8 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case F10:
 		case F11A:
 		case F11B:
+		case MUNDIAL2:
+		case MUNDIAL3:
 			datetime->year   = bcd2dec (p[6]) + 2000;
 			datetime->month  = bcd2dec (p[7]);
 			datetime->day    = bcd2dec (p[8]);
@@ -367,14 +382,16 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	unsigned int header = headersize - PAGESIZE / 2;
 	unsigned int footer = size - footersize;
 	if (parser->model == VT4 || parser->model == VT41 ||
-		parser->model == A300AI) {
+		parser->model == A300AI || parser->model == VISION ||
+		parser->model == XPAIR) {
 		header = 3 * PAGESIZE;
 	}
 
 	// Get the dive mode.
 	unsigned int mode = NORMAL;
 	if (parser->model == F10 || parser->model == F11A ||
-		parser->model == F11B) {
+		parser->model == F11B || parser->model == MUNDIAL2 ||
+		parser->model == MUNDIAL3) {
 		mode = FREEDIVE;
 	} else if (parser->model == T3B || parser->model == VT3 ||
 		parser->model == DG03) {
@@ -393,7 +410,8 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 		ngasmixes = 1;
 		o2_offset = header + 3;
 	} else if (parser->model == VT4 || parser->model == VT41 ||
-		parser->model == A300AI) {
+		parser->model == A300AI || parser->model == VISION ||
+		parser->model == XPAIR) {
 		o2_offset = header + 4;
 		ngasmixes = 4;
 	} else if (parser->model == OCI) {
@@ -482,17 +500,19 @@ oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 		switch (type) {
 		case DC_FIELD_DIVETIME:
 			if (parser->model == F10 || parser->model == F11A ||
-				parser->model == F11B)
+				parser->model == F11B || parser->model == MUNDIAL2 ||
+				parser->model == MUNDIAL3)
 				*((unsigned int *) value) = bcd2dec (data[2]) + bcd2dec (data[3]) * 60;
 			else
 				*((unsigned int *) value) = parser->divetime;
 			break;
 		case DC_FIELD_MAXDEPTH:
 			if (parser->model == F10 || parser->model == F11A ||
-				parser->model == F11B)
+				parser->model == F11B || parser->model == MUNDIAL2 ||
+				parser->model == MUNDIAL3)
 				*((double *) value) = array_uint16_le (data + 4) / 16.0 * FEET;
 			else
-				*((double *) value) = array_uint16_le (data + parser->footer + 4) / 16.0 * FEET;
+				*((double *) value) = (array_uint16_le (data + parser->footer + 4) & 0x0FFF) / 16.0 * FEET;
 			break;
 		case DC_FIELD_GASMIX_COUNT:
 			*((unsigned int *) value) = parser->ngasmixes;
@@ -568,7 +588,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	unsigned int samplerate = 1;
 	if (parser->mode != FREEDIVE) {
 		unsigned int idx = 0x17;
-		if (parser->model == A300CS || parser->model == VTX)
+		if (parser->model == A300CS || parser->model == VTX ||
+			parser->model == I450T)
 			idx = 0x1f;
 		switch (data[idx] & 0x03) {
 		case 0:
@@ -613,7 +634,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	unsigned int samplesize = PAGESIZE / 2;
 	if (parser->mode == FREEDIVE) {
 		if (parser->model == F10 || parser->model == F11A ||
-			parser->model == F11B) {
+			parser->model == F11B || parser->model == MUNDIAL2 ||
+			parser->model == MUNDIAL3) {
 			samplesize = 2;
 		} else {
 			samplesize = 4;
@@ -633,7 +655,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == ELEMENT2 || parser->model == VEO20 ||
 		parser->model == A300 || parser->model == ZEN ||
 		parser->model == GEO || parser->model == GEO20 ||
-		parser->model == MANTA) {
+		parser->model == MANTA || parser->model == I300) {
 		have_pressure = 0;
 	}
 
@@ -747,20 +769,24 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == VEO30 || parser->model == OC1A ||
 					parser->model == OC1B || parser->model == OC1C ||
 					parser->model == OCI || parser->model == A300 ||
-					parser->model == I450T) {
+					parser->model == I450T || parser->model == I300) {
 					temperature = data[offset + 3];
 				} else if (parser->model == OCS || parser->model == TX1) {
 					temperature = data[offset + 1];
-				} else if (parser->model == VT4 || parser->model == VT41 || parser->model == ATOM3 || parser->model == ATOM31 || parser->model == A300AI) {
+				} else if (parser->model == VT4 || parser->model == VT41 ||
+					parser->model == ATOM3 || parser->model == ATOM31 ||
+					parser->model == A300AI || parser->model == VISION ||
+					parser->model == XPAIR) {
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else if (parser->model == A300CS || parser->model == VTX) {
 					temperature = data[offset + 11];
 				} else {
 					unsigned int sign;
-					if (parser->model == DG03 || parser->model == PROPLUS3)
+					if (parser->model == DG03 || parser->model == PROPLUS3 ||
+						parser->model == I550T)
 						sign = (~data[offset + 5] & 0x04) >> 2;
 					else if (parser->model == VOYAGER2G || parser->model == AMPHOS ||
-						parser->model == AMPHOSAIR)
+						parser->model == AMPHOSAIR || parser->model == ZENAIR)
 						sign = (data[offset + 5] & 0x04) >> 2;
 					else if (parser->model == ATOM2 || parser->model == PROPLUS21 ||
 						parser->model == EPICA || parser->model == EPICB ||
@@ -788,7 +814,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == ATOM3 || parser->model == ATOM31 ||
 					parser->model == ZENAIR ||parser->model == A300AI ||
 					parser->model == DG03 || parser->model == PROPLUS3 ||
-					parser->model == AMPHOSAIR)
+					parser->model == AMPHOSAIR || parser->model == I550T ||
+					parser->model == VISION || parser->model == XPAIR)
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else if (parser->model == TX1 || parser->model == A300CS || parser->model == VTX)
 					pressure = array_uint16_le (data + offset + 4);
@@ -807,7 +834,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				parser->model == VEO30 || parser->model == OC1A ||
 				parser->model == OC1B || parser->model == OC1C ||
 				parser->model == OCI || parser->model == A300 ||
-				parser->model == I450T)
+				parser->model == I450T || parser->model == I300)
 				depth = (data[offset + 4] + (data[offset + 5] << 8)) & 0x0FFF;
 			else if (parser->model == ATOM1)
 				depth = data[offset + 3] * 16;
@@ -857,6 +884,15 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				decostop = data[offset + 10];
 				decotime = array_uint16_le(data + offset + 6);
 				have_deco = 1;
+			} else if (parser->model == ATOM31 || parser->model == VISION ||
+				parser->model == XPAIR) {
+				decostop = (data[offset + 5] & 0xF0) >> 4;
+				decotime = array_uint16_le(data + offset + 4) & 0x03FF;
+				have_deco = 1;
+			} else if (parser->model == I550T) {
+				decostop = (data[offset + 7] & 0xF0) >> 4;
+				decotime = array_uint16_le(data + offset + 6) & 0x03FF;
+				have_deco = 1;
 			}
 			if (have_deco) {
 				if (decostop) {
@@ -870,6 +906,26 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
 			}
 
+			unsigned int have_rbt = 0;
+			unsigned int rbt = 0;
+			if (parser->model == ATOM31) {
+				rbt = array_uint16_le(data + offset + 6) & 0x01FF;
+				have_rbt = 1;
+			} else if (parser->model == I450T) {
+				rbt = array_uint16_le(data + offset + 8) & 0x01FF;
+				have_rbt = 1;
+			} else if (parser->model == I550T) {
+				rbt = array_uint16_le(data + offset + 4) & 0x03FF;
+				have_rbt = 1;
+			} else if (parser->model == VISION || parser->model == XPAIR) {
+				rbt = array_uint16_le(data + offset + 6) & 0x03FF;
+				have_rbt = 1;
+			}
+			if (have_rbt) {
+				sample.rbt = rbt;
+				if (callback) callback (DC_SAMPLE_RBT, sample, userdata);
+			}
+
 			complete = 1;
 		}
 
