@@ -19,10 +19,6 @@
  * MA 02110-1301 USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,33 +28,19 @@
 #include "context-private.h"
 #include "device-private.h"
 #include "array.h"
+#include "usbhid.h"
 
 #ifdef _MSC_VER
 #define snprintf _snprintf
 #endif
 
-#if __APPLE__ && HAVE_HIDAPI
-#include "hidapi/hidapi.h"
-#endif
-#if HAVE_LIBUSB || (__APPLE__ && HAVE_HIDAPI)
-
-#ifdef _WIN32
-#define NOGDI
-#endif
-
-#include <libusb-1.0/libusb.h>
-
 typedef struct suunto_eonsteel_device_t {
 	dc_device_t base;
-
-	libusb_context *ctx;
-#if __APPLE__ && HAVE_HIDAPI
-	hid_device *handle;
-#else
-	libusb_device_handle *handle;
-#endif
+	dc_usbhid_t *usbhid;
 	unsigned int magic;
 	unsigned short seq;
+	unsigned char version[0x30];
+	unsigned char fingerprint[4];
 } suunto_eonsteel_device_t;
 
 // The EON Steel implements a small filesystem
@@ -88,13 +70,14 @@ struct directory_entry {
 #define READDIR_CMD    0x0910
 #define DIR_CLOSE_CMD  0x0a10
 
+static dc_status_t suunto_eonsteel_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t suunto_eonsteel_device_close(dc_device_t *abstract);
 
 static const dc_device_vtable_t suunto_eonsteel_device_vtable = {
 	sizeof(suunto_eonsteel_device_t),
 	DC_FAMILY_SUUNTO_EONSTEEL,
-	NULL, /* set_fingerprint */
+	suunto_eonsteel_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
 	NULL, /* write */
 	NULL, /* dump */
@@ -147,18 +130,13 @@ static void put_le32(unsigned int val, unsigned char *p)
 static int receive_packet(suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
 {
 	unsigned char buf[64];
-	const int InEndpoint = 0x82;
-	int rc, transferred,  len;
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	size_t transferred = 0;
+	int len;
 
-	/* 5000 = 5s timeout */
-#if __APPLE__ && HAVE_HIDAPI
-	transferred = hid_read_timeout(eon->handle, buf, PACKET_SIZE, 5000);
-	rc = (transferred <= 0) ? -1 : 0;
-#else
-	rc = libusb_interrupt_transfer(eon->handle, InEndpoint, buf, PACKET_SIZE, &transferred, 5000);
-#endif
-	if (rc) {
-		ERROR(eon->base.context, "read interrupt transfer failed (%s)", libusb_error_name(rc));
+	rc = dc_usbhid_read(eon->usbhid, buf, PACKET_SIZE, &transferred);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR(eon->base.context, "read interrupt transfer failed");
 		return -1;
 	}
 	if (transferred != PACKET_SIZE) {
@@ -188,11 +166,11 @@ static int send_cmd(suunto_eonsteel_device_t *eon,
 	unsigned int len,
 	const unsigned char *buffer)
 {
-	const int OutEndpoint = 0x02;
 	unsigned char buf[64];
-	int transferred, rc;
 	unsigned short seq = eon->seq;
 	unsigned int magic = eon->magic;
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	size_t transferred = 0;
 
 	// Two-byte packet header, followed by 12 bytes of extended header
 	if (len > sizeof(buf)-2-12) {
@@ -222,14 +200,8 @@ static int send_cmd(suunto_eonsteel_device_t *eon,
 		memcpy(buf+14, buffer, len);
 	}
 
-#if __APPLE__ && HAVE_HIDAPI
-	/* there is no write with timeout */
-	transferred = hid_write(eon->handle, buf, PACKET_SIZE);
-	rc = (transferred <= 0) ? -1 : 0;
-#else
-	rc = libusb_interrupt_transfer(eon->handle, OutEndpoint, buf, sizeof(buf), &transferred, 5000);
-#endif
-	if (rc < 0) {
+	rc = dc_usbhid_write(eon->usbhid, buf, sizeof(buf), &transferred);
+	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "write interrupt transfer failed");
 		return -1;
 	}
@@ -540,32 +512,31 @@ static int get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry *
 
 static int initialize_eonsteel(suunto_eonsteel_device_t *eon)
 {
-	const int InEndpoint = 0x82;
 	const unsigned char init[] = {0x02, 0x00, 0x2a, 0x00};
 	unsigned char buf[64];
 	struct eon_hdr hdr;
 
-	/* Get rid of any pending stale input first */
-	for (;;) {
-		int transferred;
+	dc_usbhid_set_timeout(eon->usbhid, 10);
 
-#if __APPLE__ && HAVE_HIDAPI
-		transferred = hid_read_timeout(eon->handle, buf, sizeof(buf), 10);
-		int rc = (transferred <= 0) ? -1 : 0;
-#else
-		int rc = libusb_interrupt_transfer(eon->handle, InEndpoint, buf, sizeof(buf), &transferred, 10);
-#endif
-		if (rc < 0)
+	/* Get rid of any pending stale input first */
+	/* NOTE! This will cause an annoying warning from dc_usbhid_read() */
+	for (;;) {
+		size_t transferred = 0;
+
+		dc_status_t rc = dc_usbhid_read(eon->usbhid, buf, sizeof(buf), &transferred);
+		if (rc != DC_STATUS_SUCCESS)
 			break;
 		if (!transferred)
 			break;
 	}
 
+	dc_usbhid_set_timeout(eon->usbhid, 5000);
+
 	if (send_cmd(eon, INIT_CMD, sizeof(init), init)) {
 		ERROR(eon->base.context, "Failed to send initialization command");
 		return -1;
 	}
-	if (receive_header(eon, &hdr, buf, sizeof(buf)) < 0) {
+	if (receive_header(eon, &hdr, eon->version, sizeof(eon->version)) < 0) {
 		ERROR(eon->base.context, "Failed to receive initial reply");
 		return -1;
 	}
@@ -593,68 +564,27 @@ suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, const char
 	// Set up the magic handshake fields
 	eon->magic = INIT_MAGIC;
 	eon->seq = INIT_SEQ;
+	memset (eon->version, 0, sizeof (eon->version));
+	memset (eon->fingerprint, 0, sizeof (eon->fingerprint));
 
-#if __APPLE__ && HAVE_HIDAPI
-
-	if (hid_init()) {
-		ERROR(context, "hid_init() failed");
-		status = DC_STATUS_IO;
+	status = dc_usbhid_open(&eon->usbhid, context, 0x1493, 0x0030);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR(context, "unable to open device");
 		goto error_free;
 	}
 
-	eon->handle = hid_open(0x1493, 0x0030, NULL);
-	if (!eon->handle) {
-		ERROR(context, "unable to open device");
-		hid_exit();
-		status = DC_STATUS_IO;
-		goto error_usb_exit;
-	}
-
-#else
-
-	if (libusb_init(&eon->ctx)) {
-		ERROR(context, "libusb_init() failed");
-		status = DC_STATUS_IO;
-		goto error_free;
-	}
-
-	eon->handle = libusb_open_device_with_vid_pid(eon->ctx, 0x1493, 0x0030);
-	if (!eon->handle) {
-		ERROR(context, "unable to open device");
-		status = DC_STATUS_IO;
-		goto error_usb_exit;
-	}
-
-#if defined(LIBUSB_API_VERSION) && (LIBUSB_API_VERSION >= 0x01000102)
-	libusb_set_auto_detach_kernel_driver(eon->handle, 1);
-#endif
-
-	libusb_claim_interface(eon->handle, 0);
-#endif
 	if (initialize_eonsteel(eon) < 0) {
 		ERROR(context, "unable to initialize device");
 		status = DC_STATUS_IO;
-		goto error_usb_close;
+		goto error_close;
 	}
 
 	*out = (dc_device_t *) eon;
 
 	return DC_STATUS_SUCCESS;
 
-error_usb_close:
-#if __APPLE__ && HAVE_HIDAPI
-	hid_close(eon->handle);
-#else
-	libusb_close(eon->handle);
-#endif
-
-error_usb_exit:
-#if __APPLE__ && HAVE_HIDAPI
-	hid_exit();
-#else
-	libusb_exit(eon->ctx);
-#endif
-
+error_close:
+	dc_usbhid_close(eon->usbhid);
 error_free:
 	free(eon);
 	return status;
@@ -670,6 +600,22 @@ static int count_dir_entries(struct directory_entry *de)
 	return count;
 }
 
+static dc_status_t
+suunto_eonsteel_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
+{
+	suunto_eonsteel_device_t *device = (suunto_eonsteel_device_t *) abstract;
+
+	if (size && size != sizeof (device->fingerprint))
+		return DC_STATUS_INVALIDARGS;
+
+	if (size)
+		memcpy (device->fingerprint, data, sizeof (device->fingerprint));
+	else
+		memset (device->fingerprint, 0, sizeof (device->fingerprint));
+
+	return DC_STATUS_SUCCESS;
+}
+
 static dc_status_t
 suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
@@ -679,13 +625,26 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 	dc_buffer_t *file;
 	char pathname[64];
 	unsigned int time;
+	unsigned int count = 0;
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 
 	if (get_file_list(eon, &de) < 0)
 		return DC_STATUS_IO;
 
+	// Emit a device info event.
+	dc_event_devinfo_t devinfo;
+	devinfo.model = 0;
+	devinfo.firmware = array_uint32_be (eon->version + 0x20);
+	devinfo.serial = array_convert_str2num(eon->version + 0x10, 16);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
+	count = count_dir_entries(de);
+	if (count == 0)  {
+		return DC_STATUS_SUCCESS;
+	}
+
 	file = dc_buffer_new(0);
-	progress.maximum = count_dir_entries(de);
+	progress.maximum = count;
 	progress.current = 0;
 	device_event_emit(abstract, DC_EVENT_PROGRESS, &progress);
 
@@ -693,6 +652,8 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 		int len;
 		struct directory_entry *next = de->next;
 		unsigned char buf[4];
+		const unsigned char *data = NULL;
+		unsigned int size = 0;
 
 		if (device_is_cancelled(abstract))
 			skip = 1;
@@ -719,13 +680,17 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 			rc = read_file(eon, pathname, file);
 			if (rc < 0)
 				break;
-			if (!callback)
-				break;
-			if (!callback(dc_buffer_get_data(file), dc_buffer_get_size(file), NULL, 0, userdata))
-				skip = 1;
 
-			// We've used up the buffer, so create a new one
-			file = dc_buffer_new(0);
+			data = dc_buffer_get_data(file);
+			size = dc_buffer_get_size(file);
+
+			if (memcmp (data, eon->fingerprint, sizeof (eon->fingerprint)) == 0) {
+				skip = 1;
+				break;
+			}
+
+			if (callback && !callback(data, size, data, sizeof(eon->fingerprint), userdata))
+				skip = 1;
 		}
 		progress.current++;
 		device_event_emit(abstract, DC_EVENT_PROGRESS, &progress);
@@ -743,25 +708,7 @@ suunto_eonsteel_device_close(dc_device_t *abstract)
 {
 	suunto_eonsteel_device_t *eon = (suunto_eonsteel_device_t *) abstract;
 
-
-#if __APPLE__ && HAVE_HIDAPI
-	hid_close(eon->handle);
-	hid_exit();
-#else
-	libusb_close(eon->handle);
-	libusb_exit(eon->ctx);
-#endif
+	dc_usbhid_close(eon->usbhid);
 
 	return DC_STATUS_SUCCESS;
 }
-
-#else // no LIBUSB support
-
-dc_status_t
-suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
-{
-	ERROR(context, "The Suunto EON Steel backend needs libusb-1.0");
-	return DC_STATUS_UNSUPPORTED;
-}
-
-#endif
